/* symo_qt.h
 *
 * Qt/X11-based user interface to MP3 editing
 * (K) All Rites Reversed -- Copy What You Like (see file COPYING)
 *
 * Authors:
 *     Peter Hartley      <pdh@chaos.org.uk>
 */

#ifndef included_symo_qt_h
#define included_symo_qt_h

#ifndef included_symo_h
#include "symo.h"
#endif

#include <set>
#include <map>

class QProgressDialog;
class QTextView;

/* Refuse to compile against Qt 2.1 or earlier, because we need QTable support
 */
#include <qglobal.h>
#if defined(QT_VERSION)
#if QT_VERSION < 220
#error Symo requires Qt version 2.2 or later
#endif
#else
#error Symo requires Qt version 2.2 or later
#endif

#include <qtable.h>

class SymoGrid: public QTable
{
    Q_OBJECT

public:
    SymoGrid( Symo*, QWidget *parent = NULL );
    ~SymoGrid();

    void addEntry( const Symo::Entry& );
    void updateEntry( unsigned int, const Symo::Entry& );
    void updateIndex( unsigned int );

    // virtual void swapRows( int row1, int row2 );
    virtual void columnClicked(int col);
    virtual void endEdit(int,int,bool,bool);
    virtual void keyPressEvent( QKeyEvent* );
    
    enum {
	COL_FILENAME,
	COL_EDITED,
	COL_SIZE,
	COL_MMSS,
	COL_BITRATE,
	COL_ARTIST,
	COL_ALBUM,
	COL_TRACK,
	COL_TITLE,
	COL_YEAR,
	COL_GENRE,
	COL_MTIME,
	COL_INDEX, // hidden
	NUM_COLUMNS
    };

    void fitAll();

public slots:
    void onValueChanged( int row, int col );

protected:
    void checkUpdate( int row, int col, QString text,
		      std::set<unsigned int>& needs_updating );
    void buildIndexMap();
    void doAutoFixup();

    unsigned int m_nFiles;
    QPixmap m_pmEdited, m_pmNull;
    Symo *m_symo;
    int m_lastSortCol;
    bool m_sortAsc;
    QString m_oldText; // so we can tell whether a valueChanged is real or not

    typedef std::map<unsigned int,unsigned int> map_t;
    map_t m_map;
};

class SymoWindow: public QWidget, public SymoObserver
{
    Q_OBJECT

 public:
    SymoWindow( Symo*, QWidget *parent=0 );
    ~SymoWindow();

    void closeEvent( QCloseEvent* );
    void focusInEvent( QFocusEvent* );

 public slots:
    void onOK();
    void onCancel();

 protected:
    bool okToQuit();

    SymoGrid *m_grid;
    Symo *m_symo;
    QProgressDialog *m_progress;
    QTextView *m_errors;

    // Being a SymoObserver
    bool onActivity( unsigned int, unsigned int );
    void onError( string );
    void onEntryChanged( unsigned int );
};

/** A table item that sorts numerically, not lexicographically */
class NumericTableItem: public QTableItem
{
public:
    NumericTableItem( QTable *table,
		      QTableItem::EditType et,
		      const QString& text )
	: QTableItem( table, et, text ) {}
    QString key() const;
    virtual int alignment() const { return AlignRight; }
};


/** A table item that sorts numerically, but doesn't display the actual number
 * (e.g. for time_t's). Probably makes no sense to be editable.
 */
class HiddenNumericTableItem: public QTableItem
{
    unsigned int m_value;

public:
    HiddenNumericTableItem( QTable *table,
			    QTableItem::EditType et,
			    const QString& text,
			    unsigned int value );
    QString key() const;
    virtual int alignment() const { return AlignRight; }
};

#endif
